/**
 * 
 */
package org.processmining.models.visualcnet;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;

import javax.swing.SwingConstants;

import org.processmining.models.cnet.CNet;
import org.processmining.models.cnet.CNetNode;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.shapes.Decorated;
import org.processmining.models.shapes.RoundedRect;
import org.processmining.models.visualcnet.util.TextTransformer;

/**
 * @author aadrians Oct 7, 2011
 * 
 */
public class CNetVNode extends AbstractDirectedGraphNode implements Decorated {

	// linkage to graph and node
	private final AbstractDirectedGraph<CNetVNode, CNetVEdge> graph;
	private final CNetNode node;
	private ICNetNodeDecoration decorator = null;
	protected String tooltip;
	
	public CNetVNode(AbstractDirectedGraph<CNetVNode, CNetVEdge> graph, CNetNode node, CNet net) {
		super();
		this.graph = graph;
		this.node = node;
		this.tooltip = "<html><table><tr><td colspan=\"2\"><center><strong>" + node.getLabel()
				+ "</strong></center></td></tr>" + "<tr><td><strong>In:</strong></td><td>"
				+ TextTransformer.getBindingLabelHTMLTable(net.getInputBindings(node)) + ""
				+ "<tr><td><strong>Out:</strong></td><td>"
				+ TextTransformer.getBindingLabelHTMLTable(net.getOutputBindings(node)) + "</table></html>";

		// arrange attributes
		getAttributeMap().put(AttributeMap.SHAPE, new RoundedRect());
		getAttributeMap().put(AttributeMap.LABEL, this.node.getLabel());
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		getAttributeMap().put(AttributeMap.RESIZABLE, false);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(120, 70));
		getAttributeMap().put(AttributeMap.FILLCOLOR, Color.WHITE);
		getAttributeMap().put(AttributeMap.TOOLTIP, getToolTipText());
		getAttributeMap().put(AttributeMap.LABELVERTICALALIGNMENT, SwingConstants.TOP);
		getAttributeMap().put(AttributeMap.LABELHORIZONTALALIGNMENT, SwingConstants.CENTER);
	}
	
	@SuppressWarnings("unchecked")
	public CNetVNode(CNetV graph, CNetVNode origNode) {
		this.graph = (AbstractDirectedGraph<CNetVNode, CNetVEdge>) origNode.getGraph();
		this.node = origNode.getNode();
		this.decorator = origNode.getDecorator();
	}

	/**
	 * decorate the node
	 */
	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		if (decorator != null) {
			decorator.decorate(g2d, x, y, width, height);
		}
	}

	public String getToolTipText() 
	{
		return tooltip;
	}

	public AbstractDirectedGraph<?, ?> getGraph() {
		return graph;
	}

	/**
	 * @return the decorator
	 */
	public ICNetNodeDecoration getDecorator() {
		return decorator;
	}

	/**
	 * @param decorator
	 *            the decorator to set
	 */
	public void setDecorator(ICNetNodeDecoration decorator) {
		this.decorator = decorator;
	}

	/**
	 * @return the node
	 */
	public CNetNode getNode() {
		return node;
	}

	/**
	 * get label
	 */
	@Override
	public String getLabel() {
		return node.getLabel();
	}

}
