package org.processmining.plugins.causalgraph;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.causalgraph.Relation;
import org.processmining.models.causalgraph.SimpleCausalGraph;
import org.processmining.plugins.graphviz.dot.Dot;
import org.processmining.plugins.graphviz.dot.DotEdge;
import org.processmining.plugins.graphviz.dot.DotNode;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;

public class CausalGraphDotConverter {

	private final SimpleCausalGraph causalGraph;
	private final BiMap<XEventClass, DotNode> activityToNode;
	private final Dot dot;

	public CausalGraphDotConverter(SimpleCausalGraph causalGraph) {
		super();
		this.causalGraph = causalGraph;
		this.activityToNode = HashBiMap.create();
		this.dot = new Dot();
	}

	public Dot convert() {
		for (Relation relation : causalGraph.getCausalRelations()) {
			addDependency(relation);
		}
		return dot;
	}

	private DotNode addNodeIfNeeded(XEventClass eventClass) {
		DotNode existingNode = activityToNode.get(eventClass);
		if (existingNode == null) {
			return createNode(eventClass);
		} else {
			return existingNode;
		}
	}

	protected DotNode createNode(XEventClass eventClass) {
		DotNode newNode = dot.addNode(eventClass.getId());
		newNode.setOption("shape", "box");
		activityToNode.put(eventClass, newNode);
		return newNode;
	}

	protected DotEdge addDependency(Relation relation) {
		DotNode sourceNode = addNodeIfNeeded(relation.getSource());
		DotNode targetNode = addNodeIfNeeded(relation.getTarget());
		return dot.addEdge(sourceNode, targetNode);
	}

	public BiMap<XEventClass, DotNode> getActivityToNode() {
		return activityToNode;
	}

}