package org.processmining.plugins.dcnet;

import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.RectangularShape;

final class EllipseUtils {

	private EllipseUtils() {
	}

	public static double degreeOnEllipse(double x, double y, double rx, double ry) {
		double atan2 = Math.atan2(rx * y, ry * x);
		//correct for discontinuity and for counter clockwise angle used by Arc2D
		if (atan2 < 0d) {
			return 360d - (Math.toDegrees(atan2) + 360d);
		} else {
			return 360d - (Math.toDegrees(atan2));
		}
	}

	public static double distanceFromEllipse(Point2D point, Ellipse2D ellipse) {
		double ry = ellipse.getHeight() / 2d;
		double rx = ellipse.getWidth() / 2d;
		double px = point.getX() - ellipse.getCenterX();
		double py = point.getY() - ellipse.getCenterY();
		return ((px * px) / (rx * rx)) + ((py * py) / (ry * ry)) - 1d;
	}

	public static double degreeOnEllipse(Point2D p, RectangularShape ellipseRectangle) {
		double rx = ellipseRectangle.getWidth() / 2d;
		double ry = ellipseRectangle.getHeight() / 2d;
		return degreeOnEllipse(p.getX() - ellipseRectangle.getCenterX(), p.getY() - ellipseRectangle.getCenterY(), rx,
				ry);
	}

}
